function [AT,CT,PvT,PwT, mess,lams,lamt]=SysSolveMD(A,B,C,Pv,Pw)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 
% This Code is loosely based on Gensys by Chris Sims
% The main difference is that SysSolve handles the indeterminate case.
% PvT is the impact matrix of fundamental shocks for the reduced form and PwT is the
% impact matrix of sunspot shocks. In the case of determinacy PwT is empty.
% In the case of indeterminacy more solutions can be generated by choosing
% the vcv matrix of w and v to be non-diagonal.
%
% The input system is A*y(t)=B*y*t-1)+C+Pv*v(t)+Pw*w(t)
% Output system is y(t)=AT*y(t-1)+CT+PvT*v(t)+PwT*w(t)
% 
% The fundamental shocks are assumed iid (unlike Sims' code gensys). To
% handle more complicated forcing processs; include the dynamics in 
% the vector y(t).
% 
% N.B. This program requires the m-files qzzero qzdiv qzswitch and svdsims 
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Andreas Beyer and Roger E. A. Farmer
% February 2004
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
[s t q z gev]=qz(A,B);
tiny=1e-06;
n=size(A,1);
nw=size(Pw,2);
nwc=n-nw;
nu=0;
nzero=0;

for i=1:n
    if and(abs(t(i,i))<tiny,abs(s(i,i))<tiny);
    disp('Error: Coincident Zeros');
    return;
    else
    nu=nu+(abs(t(i,i))>abs(s(i,i)));
    nzero=nzero+(abs(t(i,i))<tiny);
    end
end;

[s,t, q, z]= qzdiv(1,s,t,q,z);
gev=[diag(s)'; diag(t)'];
qv=q*Pv;qw=q*Pw;qc=q*C;
ns=n-nu;
nv=size(Pv,2);

lams=diag(s);lamt=diag(t);

qv1=qv(1:ns,:);qv2=qv(ns+1:n,:);
qw1=qw(1:ns,:);qw2=qw(ns+1:n,:);
qc1=qc(1:ns);qc2=qc(ns+1:n);
s11=s(1:ns,1:ns);t11=t(1:ns,1:ns);
s22=s(ns+1:n,ns+1:n);t22=t(ns+1:n,ns+1:n);
t12=t(1:ns,ns+1:n);s12=s(1:ns,ns+1:n);
z1=z(:,1:ns);



[uv,vv,dv,uvb,vvb]=svdsims(qv2);
[uw1,vw1,dw1,uw1b,vw1b]=svdsims(qw1);
[uw2,vw2,dw2,uw2b,vw2b]=svdsims(qw2);
existx=norm(uv-uw2*uw2'*uv) < tiny*n;
unique=norm(vw1-vw2*vw2'*vw1)<tiny*n;

if nu>nw|~existx;
    mess='System has no convergent solution';
    disp(mess);
    ind=2;  
end
  
if unique;
    ind=1;
    mess='Unique Equilibrium';
else
    mess='Indeterminate Equilibria';
    ind=0;
end
phi=qw1*vw2*inv(dw2)*uw2';

H=z*[inv(s11) -inv(s11)*(s12-phi*s22) ; zeros(nu,ns) eye(nu)];
AT=real(z1*inv(s11)*[t11 (t12-phi*t22)]*z');
CT=real(H*[qc1-phi*qc2;inv(s22-t22)*qc2]);
PvT=real(H*[qv1-phi*qv2; zeros(nu,nv)]);
PwT=real(H*[qw1*vw2b ; zeros(nu,size(vw2b,2))]);



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  END %%%%%%%%%%%%%%%%%%%%%%%%%%%%%