function [g,c,pva,pvb,gbar,cbar,pvabar,pvbbar]= convert(Gamstar,Cstar,Pvstar,Pwstar,Gamstarbar,Cstarbar,Pvstarbar,Pwstarbar)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% This takes the output of the function Findequiv and rewrites the
% reduced forms of the original and the transformed systems so that the
% state variables all appear in the top left of the system. This reordering
% may increase the number of lags appearing in the errors from 0 to 2.

% The input systems are
%
% X(t)= Gamstar*X(t-1) + Cstar + Pvstar*v(t)  
% (this is the original system)
%
% and
%
% X(t)= Gamstarbar*X(t-1) + Cstarbar + [Pvstarbar Pwstarbar]*v1(t)
% (this is the equivalent system)
%
% The output of the code is a pair of systems;
%
% x(t) = g*x(t-1) + c + pva*v(t) + pvb*v(t-1)
% (this is the transform of the original system)
%
% x(t) = gbar*x(t-1) + cbar + pvabar*v1(t) 
%        + pvbbar*v1(t-1)
% (this is the transform of the equivalent system)
%
% To check that the program is working: it should be the case that g=gs,
% c=cs, pva*pva' = pvabar*W*pvabar', pvb*pvb' = pvbbar*W*pvbbar'
%
% The dimension of v1(t) (the shocks to the equivalent system) is greater 
% than the dimension of v(t) (the shocks to the original system) if the
% equivalent model is indeterminate since in this case sunspots may
% influence the solution.  Sunspot shocks always appear after the
% fundamentals in the ordering of v1(t).
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Andreas Beyer and Roger E. A. Farmer
% February 2004:  Amended November 2004
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

 [g,c,pva,pvb,pwa,pwb]=arrangeA(Gamstar,Cstar,Pvstar,Pwstar);
 [gbar,cbar,pvabar,pvbbar,pwabar,pwbbar]=...
     arrangeA(Gamstarbar,Cstarbar,Pvstarbar,Pwstarbar);
 pvabar=[pvabar pwabar]; pvbbar=[pvbbar pwbbar];
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%   END  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 